package za.org.treehouse.hypoalarm;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.util.Log;

import java.util.Timer;
import java.util.TimerTask;

public class AlarmNotify extends Service {
    private static final Timer timer = new Timer("AlarmNotify");
    public  static final int notifyID = 1;

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public void onDestroy() {
        // Remove the notification in the notification bar
        NotificationManager nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
        nm.cancel(notifyID);
        timer.cancel();
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        final int UPDATE_INTERVAL = 10*1000; // Timer is updated six times a minute
        SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(this);

        //final String phoneNumber = sharedPref.getString(getString(R.string.PhoneNumberPref), null);
        final int gracePeriod = sharedPref.getInt(getString(R.string.GracePeriodPref), 60);
        // convert gracePeriod to milliseconds and calculate when it'll fire
        final long endTime = System.currentTimeMillis() + (gracePeriod * 60 * 1000);

        //Bitmap bm = BitmapFactory.decodeResource(getResources(), R.drawable.alarm_notification);
        Bitmap bm = BitmapFactory.decodeResource(getResources(), R.drawable.ic_launcher);
        final NotificationManager nm = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
        final Notification.Builder notification = new Notification.Builder(this)
                .setContentTitle(getString(R.string.app_name))
                .setContentText(String.format(getString(R.string.notificationText), MainActivity.MinutesToGracePeriodStr(gracePeriod)))
                .setSmallIcon(R.drawable.alarm_notification)
                .setLargeIcon(bm)
                .setOnlyAlertOnce(true)
                .setAutoCancel(true)
                .setPriority(Notification.PRIORITY_HIGH);
        //.setContentText(String.format(getString(R.string.notificationText), phoneNumber) + MainActivity.MinutesToGracePeriodStr(gracePeriod))

        // Set up dismiss action
        Intent cancellerIntent = new Intent(getBaseContext(), CancelGraceReceiver.class);
        PendingIntent cancellerPendingIntent = PendingIntent.getBroadcast(getBaseContext(), MainActivity.CANCEL_GRACE_REQUEST, cancellerIntent, 0);

        // Cancel the grace period if the user clears the notification
        notification.setDeleteIntent(cancellerPendingIntent);
        // Allow the user to cancel by clicking a "Cancel" button
        notification.addAction(android.R.drawable.ic_menu_close_clear_cancel, getString(R.string.notificationCancellation), cancellerPendingIntent);
        // Allow the user to cancel by selecting the ContentText or ContentTitle
        notification.setContentIntent(cancellerPendingIntent);

        // TODO load alert activity (without sound or vibration) on select? This would allow the user to test competence
        Intent alertActivityIntent = new Intent(this, AlarmAlertActivity.class);
        alertActivityIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK |
                Intent.FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS);
        //notification.setContentIntent(alertActivityIntent);

        nm.cancel(notifyID);
        nm.notify(notifyID, notification.build());

        timer.scheduleAtFixedRate(new TimerTask() {
            // this is called at every UPDATE_INTERVAL
            @Override
            public void run() {
                int max = 1000;
                // Convert endTime from milliseconds to seconds, and translate to time remaining
                int secondsLeft = (int) ((endTime - System.currentTimeMillis()))/(1000);
                int gracePeriodSeconds = gracePeriod * 60;
                // Multiply each int by 1000 for greater progress resolution
                int progress = ((((gracePeriodSeconds * 1000) - (secondsLeft * 1000)) * max) / (gracePeriodSeconds * 1000) );
                //Log.d("AlarmNotify", "secondsLeft is "+secondsLeft+" and progress is "+progress+" (gracePeriodSeconds is "+gracePeriodSeconds+")");
                if (secondsLeft >= 0) {
                    int minutesLeft = secondsLeft/60;
                    notification.setContentText(String.format(getString(R.string.notificationText), MainActivity.MinutesToGracePeriodStr(minutesLeft)));
                    notification.setProgress(max, progress, false);
                    // Update the notification
                    nm.notify(notifyID, notification.build());
                } else {
                    nm.cancel(notifyID);
                    this.cancel(); // Cancel timer
                    stopSelf();    // stop service
                }
            }
        }, 0, UPDATE_INTERVAL);

		return super.onStartCommand(intent, flags, startId);
    }

}
